@@ -76,19 +76,15 @@
 #include <stdio.h>
 #include <errno.h>
 #include <signal.h>
+#include <sys/io.h>
 #include <sys/ioctl.h>
-#if (__GNU_LIBRARY__ >= 6)
-#include <sys/perm.h>
-#else
-#include <linux/types.h>
-#include <linux/termios.h>
-#endif
 #include <linux/vt.h>
 #include "paths.h"
 #include "getfd.h"
 #include "findfile.h"
 #include "nls.h"
 #include "version.h"
+#include "kbd_error.h"
 
 #define MODE_RESTORETEXTMODE	0
 #define MODE_VGALINES		1
@@ -104,8 +100,8 @@ static int vga_get_fontheight(void);
 static void vga_set_cursor(int, int);
 static void vga_set_verticaldisplayend_lowbyte(int);
 
-char *dirpath[] = { "", DATADIR "/" VIDEOMODEDIR "/", 0};
-char *suffixes[] = { "", 0 };
+const char *const dirpath[] = { "", DATADIR "/" VIDEOMODEDIR "/", 0};
+const char *const suffixes[] = { "", 0 };
 
 int
 main(int argc, char **argv) {
@@ -115,14 +111,14 @@ main(int argc, char **argv) {
     struct winsize winsize;
     char *p;
     char tty[12], cmd[80], infile[1024];
-    FILE *fin;
     char *defaultfont;
+    lkfile_t fp;
 
     set_progname(argv[0]);
 
     setlocale(LC_ALL, "");
-    bindtextdomain(PACKAGE, LOCALEDIR);
-    textdomain(PACKAGE);
+    bindtextdomain(PACKAGE_NAME, LOCALEDIR);
+    textdomain(PACKAGE_NAME);
 
     if (argc < 2)
       usage();
@@ -138,7 +134,7 @@ main(int argc, char **argv) {
     	rr = atoi(argv[2]);
     }
     else
-    if (argc == 2 && (p = index(argv[1], 'x')) != 0)
+    if (argc == 2 && (p = strchr(argv[1], 'x')) != 0)
       rr = atoi(p+1);
     else if(argc == 3)
       rr = atoi(argv[2]);
@@ -148,35 +144,29 @@ main(int argc, char **argv) {
     if (mode == MODE_RESTORETEXTMODE) {
         /* prepare for: restoretextmode -r 80x25 */
         sprintf(infile, "%dx%d", cc, rr);
-        fin = findfile(infile, dirpath, suffixes);
-        if (!fin) {
-	    fprintf(stderr, _("resizecons: cannot find videomode file %s\n"),
-		    infile);
-	    exit(1);
-	}
- 	fpclose(fin);
+        if (lk_findfile(infile, dirpath, suffixes, &fp)) {
+            kbd_error(EXIT_FAILURE, 0, _("resizecons: cannot find videomode file %s\n"), infile);
+        }
+        lk_fpclose(&fp);
     }
 
     fd = getfd(NULL);
 
     if(ioctl(fd, TIOCGWINSZ, &winsize)) {
-	perror("TIOCGWINSZ");
-	exit(1);
+        kbd_error(EXIT_FAILURE, errno, "ioctl TIOCGWINSZ");
     }
 
     if (mode == MODE_VGALINES) {
-    	/* Get the number of columns. */
-    	cc = winsize.ws_col;
-    	if (rr != 25 && rr != 28 && rr !=30 && rr != 34 && rr != 36
-    	&& rr != 40 && rr != 44 && rr != 50 && rr != 60) {
-    	    fprintf(stderr, _("Invalid number of lines\n"));
-	    exit(1);
-	}
+        /* Get the number of columns. */
+        cc = winsize.ws_col;
+        if (rr != 25 && rr != 28 && rr != 30 && rr != 34 && rr != 36 &&
+            rr != 40 && rr != 44 && rr != 50 && rr != 60) {
+            kbd_error(EXIT_FAILURE, 0, _("Invalid number of lines\n"));
+        }
     }
 
     if(ioctl(fd, VT_GETSTATE, &vtstat)) {
-	perror("VT_GETSTATE");
-	exit(1);
+        kbd_error(EXIT_FAILURE, errno, "ioctl VT_GETSTATE");
     }
 
     vtsizes.v_rows = rr;
@@ -186,8 +176,7 @@ main(int argc, char **argv) {
     vga_init_io();		/* maybe only if (mode == MODE_VGALINES) */
 
     if(ioctl(fd, VT_RESIZE, &vtsizes)) {
-	perror("VT_RESIZE");
-	exit(1);
+        kbd_error(EXIT_FAILURE, errno, "ioctl VT_RESIZE");
     }
 
     if (mode == MODE_VGALINES) {
@@ -195,16 +184,16 @@ main(int argc, char **argv) {
         int scanlines_old;
         int scanlines_new;
         int fontheight;
-   	if (winsize.ws_row == 25 || winsize.ws_row == 28 ||
-   	winsize.ws_row == 36 || winsize.ws_row == 44 ||
-   	winsize.ws_row == 50)
-   	    scanlines_old = 400;
-   	else
-   	    scanlines_old = 480;
-   	if (rr == 25 || rr == 28 || rr == 36 || rr == 44 || rr == 50)
-   	    scanlines_new = 400;
-   	else
-   	    scanlines_new = 480;
+        if (winsize.ws_row == 25 || winsize.ws_row == 28 ||
+            winsize.ws_row == 36 || winsize.ws_row == 44 ||
+            winsize.ws_row == 50)
+            scanlines_old = 400;
+        else
+            scanlines_old = 480;
+        if (rr == 25 || rr == 28 || rr == 36 || rr == 44 || rr == 50)
+            scanlines_new = 400;
+        else
+            scanlines_new = 480;
         /* Switch to 400 or 480 scanline vertical timing if required. */
         if (scanlines_old != 400 && scanlines_new == 400)
             vga_400_scanlines();
@@ -221,7 +210,7 @@ main(int argc, char **argv) {
         case 50 : fontheight = 8; break;
         case 60 : fontheight = 8; break;
         default : fontheight = 8; break;
-	}
+        }
 	/* Set the VGA character height. */
 	vga_set_fontheight(fontheight);
 	/* Set the line offsets within a character cell of the cursor. */
@@ -243,13 +232,13 @@ main(int argc, char **argv) {
 
     if (mode == MODE_RESTORETEXTMODE) {
 	/* do: restoretextmode -r 25x80 */
-	sprintf(cmd, "restoretextmode -r %s\n", pathname);
+	sprintf(cmd, "restoretextmode -r %s\n", fp.pathname);
 	errno = 0;
 	if(system(cmd)) {
 	    if(errno)
 		perror("restoretextmode");
 	    fprintf(stderr, _("resizecons: the command `%s' failed\n"), cmd);
-	    exit(1);
+	    exit(EXIT_FAILURE);
 	}
     }
 
@@ -272,7 +261,7 @@ main(int argc, char **argv) {
 	  }
 	  if (fd >= 0) {
 	      if(ioctl(fd, TIOCSWINSZ, &winsize))
-		perror("TIOCSWINSZ");
+	          kbd_warning(errno, "ioctl TIOCSWINSZ");
 	      close(fd);
 	  }
       }
@@ -318,7 +307,7 @@ main(int argc, char **argv) {
 	if(errno)
 	    perror("setfont");
 	fprintf(stderr, "resizecons: the command `%s' failed\n", cmd);
-	exit(1);
+	exit(EXIT_FAILURE);
     }
 
     fprintf(stderr, _("resizecons: don't forget to change TERM "
@@ -328,17 +317,17 @@ main(int argc, char **argv) {
       fprintf(stderr,
 	      "Also the variables LINES and COLUMNS may need adjusting.\n");
 
-    return 0;
+    return EXIT_SUCCESS;
 }
 
-static void
+static void __attribute__ ((noreturn))
 usage() {
     fprintf(stderr,
 	    _("resizecons:\n"
 	      "call is:  resizecons COLSxROWS  or:  resizecons COLS ROWS\n"
 	      "or: resizecons -lines ROWS, with ROWS one of 25, 28, 30, 34,"
 	      " 36, 40, 44, 50, 60\n"));
-    exit(1);
+    exit(EXIT_FAILURE);
 }
 
 /*
@@ -349,13 +338,13 @@ usage() {
 /* Port I/O macros. Note that these are not compatible with the ones */
 /* defined in the kernel header files. */
 
-static inline void outb( int port, int value )
+static inline void my_outb( int port, int value )
 {
 	__asm__ volatile ("outb %0,%1"
 	: : "a" ((unsigned char)value), "d" ((unsigned short)port));
 }
 
-static inline int inb( int port )
+static inline int my_inb( int port )
 {
 	unsigned char value;
 	__asm__ volatile ("inb %1,%0"
@@ -373,115 +362,115 @@ static void vga_init_io() {
 	if (iopl(3) < 0) {
 		fprintf(stderr,
 			_("resizecons: cannot get I/O permissions.\n"));
-		exit(1);
+		exit(EXIT_FAILURE);
 	}
 	crtcport = 0x3d4;
-	if ((inb(0x3cc) & 0x01) == 0)
+	if ((my_inb(0x3cc) & 0x01) == 0)
 		crtcport = 0x3b4;
 }
 
 static void vga_set_fontheight( int h ) {
-	outb(crtcport, 0x09);
-	outb(crtcport + 1, (inb(crtcport + 1) & 0xe0) | (h - 1));
+	my_outb(crtcport, 0x09);
+	my_outb(crtcport + 1, (my_inb(crtcport + 1) & 0xe0) | (h - 1));
 }
 
 static int vga_get_fontheight() {
-	outb(crtcport, 0x09);
-	return (inb(crtcport + 1) & 0x1f) + 1;
+	my_outb(crtcport, 0x09);
+	return (my_inb(crtcport + 1) & 0x1f) + 1;
 }
 
 static void vga_set_cursor( int top, int bottom ) {
-	outb(crtcport, 0x0a);
-	outb(crtcport + 1, (inb(crtcport + 1) & 0xc0) | top);
-	outb(crtcport, 0x0b);
-	outb(crtcport + 1, (inb(crtcport + 1) & 0xe0) | bottom);
+	my_outb(crtcport, 0x0a);
+	my_outb(crtcport + 1, (my_inb(crtcport + 1) & 0xc0) | top);
+	my_outb(crtcport, 0x0b);
+	my_outb(crtcport + 1, (my_inb(crtcport + 1) & 0xe0) | bottom);
 }
 
 static void vga_set_verticaldisplayend_lowbyte( int byte ) {
 	/* CRTC register 0x12 */
 	/* vertical display end */
-	outb(crtcport, 0x12);
-	outb(crtcport + 1, byte);
+	my_outb(crtcport, 0x12);
+	my_outb(crtcport + 1, byte);
 }
 
 static void vga_480_scanlines() {
 	/* CRTC register 0x11 */
 	/* vertical sync end (also unlocks CR0-7) */
-	outb(crtcport, 0x11);
-	outb(crtcport + 1, 0x0c);
+	my_outb(crtcport, 0x11);
+	my_outb(crtcport + 1, 0x0c);
 
 	/* CRTC register 0x06 */
 	/* vertical total */
-	outb(crtcport, 0x06);
-	outb(crtcport + 1, 0x0b);
+	my_outb(crtcport, 0x06);
+	my_outb(crtcport + 1, 0x0b);
 
 	/* CRTC register 0x07 */
 	/* (vertical) overflow */
-	outb(crtcport, 0x07);
-	outb(crtcport + 1, 0x3e);
+	my_outb(crtcport, 0x07);
+	my_outb(crtcport + 1, 0x3e);
 
 	/* CRTC register 0x10 */
 	/* vertical sync start */
-	outb(crtcport, 0x10);
-	outb(crtcport + 1, 0xea);
+	my_outb(crtcport, 0x10);
+	my_outb(crtcport + 1, 0xea);
 
 	/* CRTC register 0x12 */
 	/* vertical display end */
-	outb(crtcport, 0x12);
-	outb(crtcport + 1, 0xdf);
+	my_outb(crtcport, 0x12);
+	my_outb(crtcport + 1, 0xdf);
 
 	/* CRTC register 0x15 */
 	/* vertical blank start */
-	outb(crtcport, 0x15);
-	outb(crtcport + 1, 0xe7);
+	my_outb(crtcport, 0x15);
+	my_outb(crtcport + 1, 0xe7);
 
 	/* CRTC register 0x16 */
 	/* vertical blank end */
-	outb(crtcport, 0x16);
-	outb(crtcport + 1, 0x04);
+	my_outb(crtcport, 0x16);
+	my_outb(crtcport + 1, 0x04);
 
 	/* Misc Output register */
 	/* Preserver clock select bits and set correct sync polarity */
-	outb(0x3c2, (inb(0x3cc) & 0x0d) | 0xe2);
+	my_outb(0x3c2, (my_inb(0x3cc) & 0x0d) | 0xe2);
 }
 
 static void vga_400_scanlines() {
 	/* CRTC register 0x11 */
 	/* vertical sync end (also unlocks CR0-7) */
-	outb(crtcport, 0x11);
-	outb(crtcport + 1, 0x0e);
+	my_outb(crtcport, 0x11);
+	my_outb(crtcport + 1, 0x0e);
 
 	/* CRTC register 0x06 */
 	/* vertical total */
-	outb(crtcport, 0x06);
-	outb(crtcport + 1, 0xbf);
+	my_outb(crtcport, 0x06);
+	my_outb(crtcport + 1, 0xbf);
 
 	/* CRTC register 0x07 */
 	/* (vertical) overflow */
-	outb(crtcport, 0x07);
-	outb(crtcport + 1, 0x1f);
+	my_outb(crtcport, 0x07);
+	my_outb(crtcport + 1, 0x1f);
 
 	/* CRTC register 0x10 */
 	/* vertical sync start */
-	outb(crtcport, 0x10);
-	outb(crtcport + 1, 0x9c);
+	my_outb(crtcport, 0x10);
+	my_outb(crtcport + 1, 0x9c);
 
 	/* CRTC register 0x12 */
 	/* vertical display end */
-	outb(crtcport, 0x12);
-	outb(crtcport + 1, 0x8f);
+	my_outb(crtcport, 0x12);
+	my_outb(crtcport + 1, 0x8f);
 
 	/* CRTC register 0x15 */
 	/* vertical blank start */
-	outb(crtcport, 0x15);
-	outb(crtcport + 1, 0x96);
+	my_outb(crtcport, 0x15);
+	my_outb(crtcport + 1, 0x96);
 
 	/* CRTC register 0x16 */
 	/* vertical blank end */
-	outb(crtcport, 0x16);
-	outb(crtcport + 1, 0xb9);
+	my_outb(crtcport, 0x16);
+	my_outb(crtcport + 1, 0xb9);
 
 	/* Misc Output register */
 	/* Preserver clock select bits and set correct sync polarity */
-	outb(0x3c2, (inb(0x3cc) & 0x0d) | 0x62);
+	my_outb(0x3c2, (my_inb(0x3cc) & 0x0d) | 0x62);
 }
