@@ -8,17 +8,16 @@
  */
 
 #include <stdio.h>
+#include <stdlib.h>
+#include <errno.h>
 #include <fcntl.h>
 #include <linux/kd.h>
 #include <sys/ioctl.h>
 #include "nls.h"
 #include "version.h"
+#include "kbd_error.h"
 
-#ifdef __sparc__
-#include <asm/kbio.h>
-#endif
-
-static void
+static void __attribute__ ((noreturn))
 usage(void)
 {
     fprintf(stderr, _(
@@ -41,7 +40,7 @@ usage(void)
     ""
 #endif
     );
-    exit(1);
+    exit(EXIT_FAILURE);
 }
 
 #define onoff(a) ((a) ? _("on ") : _("off"))
@@ -83,19 +82,18 @@ struct led {
 };
 
 static void
-getleds(char *leds) {
-    if (ioctl(0, KDGETLED, leds)) {
-	perror("KDGETLED");
-	fprintf(stderr,
-	  _("Error reading current led setting. Maybe stdin is not a VT?\n"));
-	exit(1);
+getleds(char *cur_leds) {
+    if (ioctl(0, KDGETLED, cur_leds)) {
+	kbd_error(EXIT_FAILURE, errno, _("Error reading current led setting. "
+	                                 "Maybe stdin is not a VT?: "
+	                                 "ioctl KDGETLED"));
     }
 }
 
 static int
-setleds(char leds) {
-    if (ioctl(0, KDSETLED, leds)) {
-	perror("KDSETLED");
+setleds(char cur_leds) {
+    if (ioctl(0, KDSETLED, cur_leds)) {
+	kbd_warning(errno, "ioctl KDSETLED");
 	return -1;
     }
     return 0;
@@ -104,42 +102,47 @@ setleds(char leds) {
 static void
 getflags(char *flags) {
     if (ioctl(0, KDGKBLED, flags)) {
-	perror("KDGKBLED");
-	fprintf(stderr,
-          _("Error reading current flags setting. Maybe an old kernel?\n"));
-	exit(1);
+	kbd_error(EXIT_FAILURE, errno, _("Error reading current flags setting. "
+	                                 "Maybe you are not on the console?: "
+	                                 "ioctl KDGKBLED"));
     }
 }
 
 static int sunkbdfd = -1;
 
+#ifndef KIOCGLED
+#define arg_state __attribute__ ((unused))
+#else
+#define arg_state
+#endif
+
 static void
-sungetleds(char *leds) {
+sungetleds(arg_state char *cur_leds) {
 #ifdef KIOCGLED
-    if (ioctl(sunkbdfd, KIOCGLED, leds)) {
-	perror("KIOCGLED");
-	fprintf(stderr,
-	  _("Error reading current led setting from /dev/kbd.\n"));
-	exit(1);
+    if (ioctl(sunkbdfd, KIOCGLED, cur_leds)) {
+	kbd_error(EXIT_FAILURE, errno, _("Error reading current led setting from /dev/kbd: "
+	                                 "ioctl KIOCGLED"));
     }
 #else
-    fprintf(stderr, _("KIOCGLED unavailable?\n"));
-    exit(1);
+    kbd_error(EXIT_FAILURE, 0, _("KIOCGLED unavailable?\n"));
 #endif
 }
 
+#ifndef KIOCSLED
+#define arg_state __attribute__ ((unused))
+#else
+#define arg_state
+#endif
+
 static void
-sunsetleds(char *leds) {
+sunsetleds(arg_state char *cur_leds) {
 #ifdef KIOCSLED
-    if (ioctl(sunkbdfd, KIOCSLED, leds)) {
-	perror("KIOCGSLED");
-	fprintf(stderr,
-	  _("Error reading current led setting from /dev/kbd.\n"));
-	exit(1);
+    if (ioctl(sunkbdfd, KIOCSLED, cur_leds)) {
+	kbd_error(EXIT_FAILURE, errno, _("Error reading current led setting from /dev/kbd: "
+	                                 "ioctl KIOCSLED"));
     }
 #else
-    fprintf(stderr, _("KIOCSLED unavailable?\n"));
-    exit(1);
+    kbd_error(EXIT_FAILURE, 0, _("KIOCSLED unavailable?\n"));
 #endif
 }
 
@@ -148,24 +151,22 @@ main(int argc, char **argv) {
     int optL = 0, optD = 0, optF = 0, verbose = 0;
     char oleds, nleds, oflags, nflags, odefflags, ndefflags;
     char nval, ndef, sign;
-    char osunleds, nsunleds, nsunval, nsundef;
+    char osunleds = 0, nsunleds, nsunval, nsundef;
     char *ap;
     struct led *lp;
 
     set_progname(argv[0]);
 
     setlocale(LC_ALL, "");
-    bindtextdomain(PACKAGE, LOCALEDIR);
-    textdomain(PACKAGE);
+    bindtextdomain(PACKAGE_NAME, LOCALEDIR);
+    textdomain(PACKAGE_NAME);
 
-    if (argc == 2 && !strcmp("-V", argv[1]))
+    if (argc == 2 && (!strcmp("-V", argv[1]) || !strcmp("--version", argv[1])))
 	print_version_and_exit();
 
 #ifdef __sparc__
-    sunkbdfd = open("/dev/kbd", O_RDONLY);
-    if (sunkbdfd < 0) {
-	perror("/dev/kbd");
-	fprintf(stderr, _("Error opening /dev/kbd.\n"));
+    if ((sunkbdfd = open("/dev/kbd", O_RDONLY)) < 0) {
+	kbd_error(EXIT_FAILURE, errno, "open /dev/kbd");
 	/* exit(1); */
     }
 #endif
@@ -197,8 +198,7 @@ main(int argc, char **argv) {
 	if (optL) {
 	    nleds = 0xff;
 	    if (setleds(nleds)) {
-		fprintf(stderr, _("Error resetting ledmode\n"));
-		exit(1);
+		kbd_error(EXIT_FAILURE, 0, _("Error resetting ledmode\n"));
 	    }
 	}
 
@@ -220,7 +220,7 @@ main(int argc, char **argv) {
 	    else
 	        report(oleds);
 	}
-	exit(0);
+	exit(EXIT_SUCCESS);
     }
 
     if (!optL)
@@ -239,7 +239,7 @@ main(int argc, char **argv) {
 	    sign = 0;
 	    ap++;
 	}
-	for (lp = leds; lp-leds < sizeof(leds)/sizeof(leds[0]); lp++) {
+	for (lp = leds; (unsigned) (lp-leds) < sizeof(leds)/sizeof(leds[0]); lp++) {
 	    if(!strcmp(ap, lp->name)) {
 		if(sign) {
 		  nval |= lp->bit;
@@ -276,8 +276,7 @@ main(int argc, char **argv) {
     }
     if (optD || optF) {
 	if (ioctl(0, KDSKBLED, (ndefflags << 4) | nflags)) {
-	    perror("KDSKBLED");
-	    exit(1);
+	    kbd_error(EXIT_FAILURE, errno, "ioctl KDSKBLED");
 	}
     }
     if (optL) {
@@ -299,8 +298,8 @@ main(int argc, char **argv) {
 	        report(nleds);
 	    }
 	    if (setleds(nleds))
-	        exit(1);
+	        exit(EXIT_FAILURE);
 	}
     }
-    exit(0);
+    exit(EXIT_SUCCESS);
 }
