@@ -5,13 +5,13 @@
 #include <sys/ioctl.h>
 #include <linux/kd.h>
 #include <errno.h>
-#include <error.h>
 #include "kbd.h"
 #include "getfd.h"
 #include "nls.h"
 #include "version.h"
+#include "kbd_error.h"
 
-static unsigned char *cmap;
+static unsigned char cmap[3 * 16];
 
 /* Standard VGA terminal colors, matching those hardcoded in the Linux kernel's
  * drivers/char/vt.c
@@ -53,62 +53,45 @@ usage(int code)
 	exit(code);
 }
 
-static void
-set_colormap(unsigned char *colormap)
-{
-	int fd = getfd(NULL);
-
-	/* Apply the color map to the tty via ioctl */
-	if (ioctl(fd, PIO_CMAP, colormap) == -1)
-		error(EXIT_FAILURE, errno, "ioctl");
-
-	close(fd);
-}
-
 static void
 parse_file(FILE *fd, const char *filename)
 {
 	int c;
 	unsigned int rows, cols, val;
 
-	if ((cmap = calloc(3 * 16, sizeof(unsigned char))) == NULL)
-		error(EXIT_FAILURE, errno, "calloc");
-
 	for (rows = 0; rows < 3; rows++) {
-		cols = 0;
-
-		while (cols < 16) {
+		for (cols = 0; cols < 16; cols++) {
 			if ((c = fscanf(fd, "%u", &val)) != 1) {
 				if (c == EOF)
-					error(EXIT_FAILURE, errno, "fscanf");
+					kbd_error(EXIT_FAILURE, errno, "fscanf");
 
-				error(EXIT_FAILURE, 0, _("Error: %s: Invalid value in field %u in line %u."),
+				kbd_error(EXIT_FAILURE, 0, _("Error: %s: Invalid value in field %u in line %u."),
 				      filename, rows + 1, cols + 1);
 			}
 
 			cmap[rows + cols * 3] = (unsigned char) val;
 
 			if (cols < 15 && fgetc(fd) != ',')
-				error(EXIT_FAILURE, 0, _("Error: %s: Insufficient number of fields in line %u."),
+				kbd_error(EXIT_FAILURE, 0, _("Error: %s: Insufficient number of fields in line %u."),
 				      filename, rows + 1);
-			cols++;
 		}
 
 		if ((c = fgetc(fd)) == EOF)
-			error(EXIT_FAILURE, 0, _("Error: %s: Line %u has ended unexpectedly.\n"),
+			kbd_error(EXIT_FAILURE, 0, _("Error: %s: Line %u has ended unexpectedly.\n"),
 			      filename, rows + 1); 
 
 		if (c != '\n')
-			error(EXIT_FAILURE, 0, _("Error: %s: Line %u is too long.\n"),
+			kbd_error(EXIT_FAILURE, 0, _("Error: %s: Line %u is too long.\n"),
 			      filename, rows + 1);
 	}
 }
 
 int
 main(int argc, char **argv) {
-	int c;
+	int c, fd;
 	const char *file;
-	FILE *fd;
+	unsigned char *colormap = cmap;
+	FILE *f;
 
 	set_progname(argv[0]);
 
@@ -133,22 +116,26 @@ main(int argc, char **argv) {
 	file = argv[optind];
 
 	if (!strcmp(file, "vga")) {
-		set_colormap(vga_colors);
-		return EXIT_SUCCESS;
+		colormap = vga_colors;
 
 	} else if (!strcmp(file, "-")) {
 		parse_file(stdin, "stdin");
 
 	} else {
-		if ((fd = fopen(file, "r")) == NULL)
-			error(EXIT_FAILURE, errno, "fopen");
+		if ((f = fopen(file, "r")) == NULL)
+			kbd_error(EXIT_FAILURE, errno, "fopen");
 
-		parse_file(fd, file);
-		fclose(fd);
+		parse_file(f, file);
+		fclose(f);
 	}
 
-	set_colormap(cmap);
-	free(cmap);
+	fd = getfd(NULL);
+
+	/* Apply the color map to the tty via ioctl */
+	if (ioctl(fd, PIO_CMAP, colormap) == -1)
+		kbd_error(EXIT_FAILURE, errno, "ioctl");
+
+	close(fd);
 
 	return EXIT_SUCCESS;
 }
