@@ -1,13 +1,16 @@
 #include <stdio.h>
+#include <stdlib.h>
+#include <errno.h>
 #include <unistd.h>
 #include <getopt.h>
-#include <signal.h>
 #include <fcntl.h>
+#include <signal.h>
 #include <termios.h>
+#include <sys/ioctl.h>
 #include <linux/kd.h>
 #include <linux/keyboard.h>
-#include <sys/ioctl.h>
 #include "getfd.h"
+#include "kbd_error.h"
 #include "nls.h"
 #include "version.h"
 
@@ -26,8 +29,7 @@ get_mode(void) {
         char *m;
 
 	if (ioctl(fd, KDGKBMODE, &oldkbmode)) {
-		perror("KDGKBMODE");
-		exit(1);
+		kbd_error(EXIT_FAILURE, errno, "ioctl KDGKBMODE");
 	}
 	switch(oldkbmode) {
 	  case K_RAW:
@@ -52,28 +54,27 @@ get_mode(void) {
 static void
 clean_up(void) {
 	if (ioctl(fd, KDSKBMODE, oldkbmode)) {
-		perror("KDSKBMODE");
-		exit(1);
+		kbd_error(EXIT_FAILURE, errno, "ioctl KDSKBMODE");
 	}
 	if (tcsetattr(fd, 0, &old) == -1)
-		perror("tcsetattr");
+		kbd_warning(errno, "tcsetattr");
 	close(fd);
 }
 
-static void
+static void __attribute__ ((noreturn))
 die(int x) {
 	printf(_("caught signal %d, cleaning up...\n"), x);
 	clean_up();
-	exit(1);
+	exit(EXIT_FAILURE);
 }
 
-static void
-watch_dog(int x) {
+static void __attribute__ ((noreturn))
+watch_dog(int x __attribute__ ((unused))) {
 	clean_up();
-	exit(0);
+	exit(EXIT_SUCCESS);
 }
 
-static void
+static void __attribute__ ((noreturn))
 usage(void) {
 	fprintf(stderr, _(
 "showkey version %s\n\n"
@@ -85,8 +86,8 @@ usage(void) {
 "	-a --ascii	display the decimal/octal/hex values of the keys\n"
 "	-s --scancodes	display only the raw scan-codes\n"
 "	-k --keycodes	display only the interpreted keycodes (default)\n"
-), VERSION);
-	exit(1);
+), PACKAGE_VERSION);
+	exit(EXIT_FAILURE);
 }
 
 int
@@ -105,32 +106,32 @@ main (int argc, char *argv[]) {
 	int print_ascii = 0;
 
 	struct termios new;
-	unsigned char buf[16];
+	unsigned char buf[18];	/* divisible by 3 */
 	int i, n;
 
 	set_progname(argv[0]);
 
 	setlocale(LC_ALL, "");
-	bindtextdomain(PACKAGE, LOCALEDIR);
-	textdomain(PACKAGE);
+	bindtextdomain(PACKAGE_NAME, LOCALEDIR);
+	textdomain(PACKAGE_NAME);
 
 	while ((c = getopt_long(argc, argv,
-		short_opts, long_opts, NULL)) != -1) {
+				short_opts, long_opts, NULL)) != -1) {
 		switch (c) {
-			case 's':
-				show_keycodes = 0;
-				break;
-			case 'k':
-				show_keycodes = 1;
-				break;
-			case 'a':
-				print_ascii = 1;
-				break;
-			case 'V':
-				print_version_and_exit();
-			case 'h':
-			case '?':
-				usage();
+		case 's':
+			show_keycodes = 0;
+			break;
+		case 'k':
+			show_keycodes = 1;
+			break;
+		case 'a':
+			print_ascii = 1;
+			break;
+		case 'V':
+			print_version_and_exit();
+		case 'h':
+		case '?':
+			usage();
 		}
 	}
 
@@ -142,9 +143,9 @@ main (int argc, char *argv[]) {
 	        fd = 0;
 
 		if (tcgetattr(fd, &old) == -1)
-			perror("tcgetattr");
+			kbd_warning(errno, "tcgetattr");
 		if (tcgetattr(fd, &new) == -1)
-			perror("tcgetattr");
+			kbd_warning(errno, "tcgetattr");
 
 		new.c_lflag &= ~ (ICANON | ISIG);
 		new.c_lflag |= (ECHO | ECHOCTL);
@@ -153,7 +154,7 @@ main (int argc, char *argv[]) {
 		new.c_cc[VTIME] = 0;
 
 		if (tcsetattr(fd, TCSAFLUSH, &new) == -1)
-			perror("tcgetattr");
+			kbd_warning(errno, "tcgetattr");
 		printf(_("\nPress any keys - "
 		         "Ctrl-D will terminate this program\n\n"));
 
@@ -167,8 +168,8 @@ main (int argc, char *argv[]) {
 		}
 
 		if (tcsetattr(fd, 0, &old) == -1)
-			perror("tcsetattr");
-		exit(0);
+			kbd_warning(errno, "tcsetattr");
+		return EXIT_SUCCESS;
 	}
 
 
@@ -178,8 +179,8 @@ main (int argc, char *argv[]) {
 	signal(SIGALRM, watch_dog);
 
 	/*
-		if we receive a signal, we want to exit nicely, in
-		order not to leave the keyboard in an unusable mode
+	  if we receive a signal, we want to exit nicely, in
+	  order not to leave the keyboard in an unusable mode
 	*/
 	signal(SIGHUP, die);
 	signal(SIGINT, die);
@@ -207,9 +208,9 @@ main (int argc, char *argv[]) {
 
 	get_mode();
 	if (tcgetattr(fd, &old) == -1)
-		perror("tcgetattr");
+		kbd_warning(errno, "tcgetattr");
 	if (tcgetattr(fd, &new) == -1)
-		perror("tcgetattr");
+		kbd_warning(errno, "tcgetattr");
 
 	new.c_lflag &= ~ (ICANON | ECHO | ISIG);
 	new.c_iflag = 0;
@@ -217,29 +218,51 @@ main (int argc, char *argv[]) {
 	new.c_cc[VTIME] = 1;	/* 0.1 sec intercharacter timeout */
 
 	if (tcsetattr(fd, TCSAFLUSH, &new) == -1)
-		perror("tcsetattr");
+		kbd_warning(errno, "tcsetattr");
 	if (ioctl(fd, KDSKBMODE, show_keycodes ? K_MEDIUMRAW : K_RAW)) {
-		perror("KDSKBMODE");
-		exit(1);
+		kbd_error(EXIT_FAILURE, errno, "ioctl KDSKBMODE");
 	}
 
 	printf(_("press any key (program terminates 10s after last keypress)...\n"));
+
+	/* show scancodes */
+	if (!show_keycodes) {
+		while (1) {
+			alarm(10);
+			n = read(fd, buf, sizeof(buf));
+			for (i = 0; i < n; i++)
+				printf("0x%02x ", buf[i]);
+			printf("\n");
+		}
+		clean_up();
+		return EXIT_SUCCESS;
+	}
+
+	/* show keycodes - 2.6 allows 3-byte reports */
 	while (1) {
 		alarm(10);
 		n = read(fd, buf, sizeof(buf));
-		for (i = 0; i < n; i++) {
-			if (!show_keycodes)
-				printf("0x%02x ", buf[i]);
-			else
-				printf(_("keycode %3d %s\n"),
-					buf[i] & 0x7f,
-					buf[i] & 0x80 ? _("release")
-				                      : _("press"));
+		i = 0;
+		while (i < n) {
+			int kc;
+			char *s;
+
+			s = (buf[i] & 0x80) ? _("release") : _("press");
+
+			if (i+2 < n && (buf[i] & 0x7f) == 0
+				&& (buf[i+1] & 0x80) != 0
+				&& (buf[i+2] & 0x80) != 0) {
+				kc = ((buf[i+1] & 0x7f) << 7) |
+					(buf[i+2] & 0x7f);
+				i += 3;
+			} else {
+				kc = (buf[i] & 0x7f);
+				i++;
+			}
+			printf(_("keycode %3d %s\n"), kc, s);
 		}
-		if (!show_keycodes)
-			printf("\n");
 	}
 
 	clean_up();
-	exit(0);
+	return EXIT_SUCCESS;
 }
