@@ -9,6 +9,7 @@
  * start getty, not openvt, or anybody will have a root shell
  * with a single keystroke.
  */
+#include <stdlib.h>
 #include <signal.h>
 #include <errno.h>
 #include <linux/kd.h>
@@ -18,21 +19,24 @@
 #include <sys/ioctl.h>	/* ioctl */
 #include <unistd.h>	/* sleep */
 
+#include "version.h"
 #include "kbd.h"
+#include "kbd_error.h"
 
 static void
 sighup(int n __attribute__ ((unused))) {
     if (system("openvt -s -l bash") == -1) {
-      perror("system");
-      exit(1);
+      kbd_error(EXIT_FAILURE, errno, "system");
     }
     signal(SIGHUP, sighup);
 }
 
 int
-main(void) {
+main(int argc __attribute__ ((unused)), char *argv[]) {
     int fd;
 
+    set_progname(argv[0]);
+
     fd = open("/dev/tty0", 0);
     if (fd < 0 && errno == ENOENT)
       fd = open("/dev/vc/0", 0);
@@ -40,9 +44,9 @@ main(void) {
       fd = 0;
     signal(SIGHUP, sighup);
     if (ioctl(fd, KDSIGACCEPT, (long) SIGHUP)) {
-      perror("KDSIGACCEPT");
-      exit(1);
+      kbd_error(EXIT_FAILURE, errno, "ioctl KDSIGACCEPT");
     }
     while(1)
       sleep(3600);
+    return EXIT_SUCCESS;
 }
